﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CoveringArrayAlgorithm;
using MVVM;

namespace CoveringArray.ViewModel
{
    public class CoveringArrayStatisticsViewModel : ViewModelBase
    {
        #region properties
        public int BestResult
        {
            get { return _BestResult; }
            set
            {
                _BestResult = value;
                NotifyPropertyChanged("BestResult");
            }
        } private int _BestResult;

        public int AverageResult
        {
            get { return _AverageResult; }
            set
            {
                _AverageResult = value;
                NotifyPropertyChanged("AverageResult");
            }
        } private int _AverageResult;

        public int WorstResult
        {
            get { return _WorstResult; }
            set
            {
                _WorstResult = value;
                NotifyPropertyChanged("WorstResult");
            }
        } private int _WorstResult;

        public TimeSpan AverageTime
        {
            get { return _AverageTime; }
            set
            {
                _AverageTime = value;
                NotifyPropertyChanged("AverageTime");
            }
        } private TimeSpan _AverageTime;

        public TestSuiteCollection TestSuites
        {
            get { return _TestSuites; }
            set
            {
                _TestSuites = value;
                CalculateStatistics();
                NotifyPropertyChanged("TestSuites");
            }
        } private TestSuiteCollection _TestSuites;
        #endregion

        #region Functions
        public void Clear()
        {
            BestResult = 0;
            WorstResult = 0;
            AverageResult = 0;
            AverageTime = TimeSpan.Zero;
        }

        private void CalculateStatistics()
        {
            BestResult = TestSuites.Select(testSuite => testSuite.Tests.Count).Min();
            AverageResult = TestSuites.Sum(testSuite => testSuite.Tests.Count) / TestSuites.Count;
            WorstResult = TestSuites.Select(testSuite => testSuite.Tests.Count).Max();
            AverageTime = new TimeSpan(TestSuites.Aggregate(TimeSpan.Zero, (current, testSuite) => current + testSuite.CreationTime).Ticks / TestSuites.Count);
            NotifyPropertyChangedAll(null);
        }

        protected override void NotifyPropertyChangedAll(object inOjbect)
        {
            NotifyPropertyChanged("BestResult");
            NotifyPropertyChanged("AverageTime");
            NotifyPropertyChanged("WorstResult");
            NotifyPropertyChanged("AverageTime");
        }
        #endregion
    }
}
