﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Input;
using MVVM;
using CoveringArrayAlgorithm;
using Component = CoveringArrayAlgorithm.Component;

namespace CoveringArray.ViewModel
{
    class CoveringArrayViewModel : ViewModelBase
    {
        #region Member Variables
        #endregion

        #region Constructors
        /// <summary>
        /// The default constructor
        /// </summary>
        public CoveringArrayViewModel()
        {
            // This must be first
            _TestSuites = new TestSuiteCollection();
            TestSuites.Add(new TestSuite() { Id = 1 });
            _TestSuiteId = 0;

            // Set defaults
            NWay = 2;
            NumberOfComponents = 3;
            NumberOfOptions = 4;
            NumberOfTestSuites = 100;

            // Initialize Worker
            ProgressBarVM.Worker.RunWorkerCompleted += WorkerRunWorkerCompleted;
            ProgressBarVM.Worker.WorkerReportsProgress = true;
            ProgressBarVM.Worker.ProgressChanged += WorkerProgressChanged;
            ProgressBarVM.Worker.DoWork += WorkerDoWork;
        }
        #endregion

        #region Properties
        public TestSuiteCollection TestSuites
        {
            get { return _TestSuites; }
        } private readonly TestSuiteCollection _TestSuites;

        public int TestSuiteId
        {
            get { return _TestSuiteId; }
            set
            {
                _TestSuiteId = value;
                NotifyPropertyChanged("TestSuiteId");
                ChangeTestSuite();

            }
        } private int _TestSuiteId;

        public TestSuite TestSuite
        {
            get { return (_TestSuites.Count > 0) ? _TestSuites[TestSuiteId] : null; }
        }

        public int NumberOfTestSuites
        {
            get { return _NumberOfTestSuites; }
            set
            {
                _NumberOfTestSuites = value;
                ProgressBarVM.Max = _NumberOfTestSuites;
                NotifyPropertyChanged("NumberOfTestSuites");
            }

        } private int _NumberOfTestSuites;

        /// <summary>
        /// The NWay value. This is the number of Options grouped together
        /// where only one Option per component exists per group.
        /// </summary>
        public int NWay
        {
            get { return TestSuite.NWay; }
            set
            {
                TestSuite.NWay = value;
                NotifyPropertyChanged("NWay");
            }
        }

        /// <summary>
        /// The NWay value. This is the number of Options grouped together
        /// where only one Option per component exists per group.
        /// </summary>
        public int[] NWayOptions
        {
            get { return _NWayOptions; }
        } private readonly int[] _NWayOptions = new int[] { 2, 3, 4, 5 };

        /// <summary>
        /// A reference the the Components in the TestSuite
        /// </summary>
        public Components Components
        {
            get { return TestSuite.Components; }
            set
            {
                TestSuite.Components = value;
                NotifyPropertyChanged("Components");
            }
        }

        /// <summary>
        /// The number of components to add. This is not the number
        /// of components already added.
        /// </summary>
        public int NumberOfComponents
        {
            get { return _NumberOfComponents; }
            set
            {
                _NumberOfComponents = value;
                NotifyPropertyChanged("NumberOfComponents");
            }
        } private int _NumberOfComponents;

        /// <summary>
        /// The number of Options to add. This is not the number
        /// of Options already added.
        /// </summary>
        public int NumberOfOptions
        {
            get { return _NumberOfOptions; }
            set
            {
                _NumberOfOptions = value;
                NotifyPropertyChanged("NumberOfOptions");
            }
        } private int _NumberOfOptions;

        public TestCases Tests
        {
            get { return TestSuite.Tests; }
            set
            {
                TestSuite.Tests = value;
                NotifyPropertyChanged("Tests");
            }
        }

        /// <summary>
        /// An ICommand reference to the AddComponents function.
        /// </summary>
        public ICommand AddComponentsCommand
        {
            get
            {
                if (null == _AddComponentsCommand)
                    _AddComponentsCommand = new RelayCommand(f => AddComponents(),
                        f => AddComponentsCommandCanExecute());

                return _AddComponentsCommand;
            }
            set
            {
                _AddComponentsCommand = (RelayCommand)value;
                NotifyPropertyChanged("AddComponentsCommand");
            }
        } private RelayCommand _AddComponentsCommand;

        /// <summary>
        /// An ICommand reference to the AddComponents function.
        /// </summary>
        public ICommand CreateTestSuiteCollectionCommand
        {
            get
            {
                if (null == _CreateTestSuiteCollectionCommand)
                    _CreateTestSuiteCollectionCommand = new RelayCommand(f => CreateTestSuiteCollection(),
                        f => CreateTestSuiteCollectionCommandCanExecute());

                return _CreateTestSuiteCollectionCommand;
            }
            set
            {
                _CreateTestSuiteCollectionCommand = (RelayCommand)value;
                NotifyPropertyChanged("CreateTestSuiteCollectionCommand");
            }
        } private RelayCommand _CreateTestSuiteCollectionCommand;

        /// <summary>
        /// An ICommand reference to the AddComponents function.
        /// </summary>
        public ICommand ClearCommand
        {
            get
            {
                if (null == _ClearCommand)
                    _ClearCommand = new RelayCommand(f => Clear(),
                        f => ClearCommandCanExecute());

                return _ClearCommand;
            }
            set
            {
                _ClearCommand = (RelayCommand)value;
                NotifyPropertyChanged("CreateTestSuiteCollectionCommand");
            }
        } private RelayCommand _ClearCommand;

        public ProgressBarViewModel ProgressBarVM
        {
            get { return _ProgressBarViewModel ?? (_ProgressBarViewModel = new ProgressBarViewModel() { Min = 0, Max = NumberOfTestSuites }); }
        } private ProgressBarViewModel _ProgressBarViewModel;

        private bool IsInProgress
        {
            get { return _IsInProgress; }
            set { _IsInProgress = value; }
        } private bool _IsInProgress;

        public CoveringArrayStatisticsViewModel StatisticsVM
        {
            get { return _StatisticsVM ?? (_StatisticsVM = new CoveringArrayStatisticsViewModel()); }
        } private CoveringArrayStatisticsViewModel _StatisticsVM;

        #endregion

        #region Functions
        private void ChangeTestSuite()
        {
            NotifyPropertyChanged("TestSuite");
            NotifyPropertyChanged("Components");
            NotifyPropertyChanged("Tests");
        }

        public void AddComponents()
        {
            for (int i = 0; i < NumberOfComponents; i++)
            {
                var Options = new Component();
                Components.Add(Options);
                for (int j = 0; j < NumberOfOptions; j++)
                {
                    Options.Add(TestSuite.NextOptionId++);
                }
            }
        }

        public bool AddComponentsCommandCanExecute()
        {
            return NumberOfComponents > 0 && NumberOfOptions > 0;
        }

        public void CreateTestSuiteCollection()
        {
            IsInProgress = true;
            ProgressBarVM.Message = "Processing Started.";
            ProgressBarVM.Worker.RunWorkerAsync();
        }

        void WorkerDoWork(object sender, DoWorkEventArgs e)
        {
            TestSuiteCollectionCreator.Create(TestSuites, TestSuite, NumberOfTestSuites, sender as BackgroundWorker);
        }

        void WorkerProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ProgressBarVM.Value = e.ProgressPercentage;
            ProgressBarVM.Message = String.Format("Processing Test {0}", e.ProgressPercentage);
        }

        void WorkerRunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            NotifyPropertyChanged("TestSuites");
            NotifyPropertyChanged("Tests");
            StatisticsVM.TestSuites = TestSuites;
            ProgressBarVM.Message = "Processing Complete!";
            IsInProgress = false;
            CommandManager.InvalidateRequerySuggested();
        }

        public bool CreateTestSuiteCollectionCommandCanExecute()
        {
            return !IsInProgress && TestSuites.Count > 0 && TestSuite.Components.Count > 0;
        }

        private void Clear()
        {
            TestSuites.Clear();
            Component.NextId = 1;
            TestSuites.Add(new TestSuite() { Id = 1 });
            ProgressBarVM.Value = 0;
            StatisticsVM.Clear();
            NotifyPropertyChanged("TestSuites");
            TestSuiteId = 0;
            CommandManager.InvalidateRequerySuggested();
        }

        private bool ClearCommandCanExecute()
        {
            return !IsInProgress && TestSuites.Count > 0 && TestSuite.Components.Count > 0;
        }
        #endregion
    }
}
