﻿using System.Collections.Generic;
using System.Linq;

namespace CoveringArrayAlgorithm
{
    public class SortedTuple : List<int>
    {
        #region Constructors
        public SortedTuple() { }

        public SortedTuple(SortedTuple inSortedTouple)
        {
            if (inSortedTouple == null) return;
            for (int i = 0; i < inSortedTouple.Count; i++)
            {
                var intVal = inSortedTouple[i];
                Add(intVal);
            }
        }

        #endregion

        #region Properties
        #endregion

        #region Functions
        /// <summary>
        /// Equals override to compair values not memory reference.
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(object obj)
        {
            // Check for null values and compare run-time types.
            if (obj == null || GetType() != obj.GetType())
                return false;

            var right = obj as SortedTuple;

            return this == right;
        }

        /// <summary>
        /// Hash override. Two null values returns a hash of zero.
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            if (_HashCode == 0)
            {
                _HashCode = Count;
                for (int i = 0; i < this.Count; i++)
                {
                    _HashCode *= this[i].GetHashCode();
                }
            }

            return _HashCode;
        } private int _HashCode;

        public static bool operator ==(SortedTuple left, SortedTuple right)
        {
            // If both are null, or both are same instance, return true.
            if (System.Object.ReferenceEquals(left, right))
            {
                return true;
            }

            // If one is null, but not both, return false.
            if (((object)left == null) || ((object)right == null))
            {
                return false;
            }

            // Return false if they aren't the same size
            if (left.Count != right.Count)
                return false;

            // Compare values but expect them to be sorted
            for (int i = 0; i < left.Count; i++)
            {
                if (left[i] != right[i])
                {
                    return false;
                }
            }
            return true;
        }

        public static bool operator !=(SortedTuple left, SortedTuple right)
        {
            return !(left == right);
        }

        public override string ToString()
        {
            var retVal = string.Empty;
            foreach (var value in this)
            {
                if (retVal != string.Empty)
                    retVal += ", ";
                retVal += value.ToString();
            }
            return retVal;
        }

        new public void Add(int t)
        {
            _HashCode = 0;
            Add(t, false);
        }

        public void Add(int t, bool inOrder = false)
        {
            _HashCode = 0;
            // If it is the first one, just add it
            if (inOrder || Count == 0)
            {
                base.Add(t);
                return;
            }

            // Find where it belongs and add it there
            for (var i = 0; i < Count; i++)
            {
                if (t < this[i])
                {
                    Insert(i, t);
                    return;
                }
            }

            // If value is greater than all values, 
            // just add it at the end.
            base.Add(t);
        }
        #endregion
    }
}
