﻿using System;
using System.Linq;

namespace CoveringArrayAlgorithm
{
    public class TestSuite
    {
        public static int NextOptionId;

        /// <summary>
        /// The default constructor.
        /// </summary>
        public TestSuite()
        {
            Components = new Components();
            Tests = new TestCases();
            OptionGroups = new OptionGroups();
            UnmarkedOptionGroups = new UnmarkedOptionGroups();
            NWay = 2;
        }

        /// <summary>
        /// The Id of this TestSuite
        /// </summary>
        public int Id { get; set; }

        /// <summary>
        /// The number of Components to group together.
        /// 2-Way, 3-Way, N-Way.
        /// </summary>
        public int NWay { get; set; }

        /// <summary>
        /// This is a list of all Components
        /// </summary>
        public Components Components { get; set; }

        /// <summary>
        /// This is is a list of tests that cover the options.
        /// </summary>
        public TestCases Tests { get; set; }

        /// <summary>
        /// A field to track how long it took to make create this TestSuite
        /// </summary>
        public TimeSpan CreationTime { get; set; }

        /// <summary>
        /// This is the groups of Component options. The number in the 
        /// gorup is based on the NWay.
        /// </summary>
        public OptionGroups OptionGroups { get; set; }

        /// <summary>
        /// This is the groups of Component options. The number in the 
        /// gorup is based on the NWay.
        /// </summary>
        public UnmarkedOptionGroups UnmarkedOptionGroups { get; set; }

        public void Clear()
        {
            NextOptionId = 0;
            Component.NextId = 1;
            NWay = 2;
            Components.Clear();
            Tests.Clear();
            OptionGroups.Clear();
        }

        public void AddTestCase(SortedTuple inSortedTouple, bool inMarkOptionGroups = true)
        {
            var tc = inSortedTouple as TestCase;
            if (null == tc) // if it is not a test case, make it one
            {
                tc = new TestCase();
                foreach (var num in inSortedTouple)
                {
                    tc.Add(num);
                }
            }
            Tests.Add(tc);
            tc.GetOptionGroups(NWay);
            if (inMarkOptionGroups)
                MarkOptionGroups(tc);
        }

        private void MarkOptionGroups(TestCase inTestCase)
        {
            foreach (var key in inTestCase.OptionGroups.Keys)
            {
                OptionGroups[key] = true;
                UnmarkedOptionGroups.Remove(key);
            }
        }

        public void SetUnmarkedOptionGroups()
        {
            foreach (var optionGroup in OptionGroups)
            {
                if (!optionGroup.Value)
                    UnmarkedOptionGroups.Add(optionGroup.Key);
            }
        }
    }
}
