﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;

namespace CoveringArrayAlgorithm
{
    public class TestSuiteCreator
    {
        public static void CreateTestSuite(TestSuite inTestSuite, bool inUseExitingOptions = true)
        {
            // Do nothing if there is nothing to do.
            if (null == inTestSuite || null == inTestSuite.Components || inTestSuite.Components.Count == 0)
                return;

            // Clear any existing test cases
            inTestSuite.Tests.Clear();

            // Create pairs or N-Way groups
            if (!inUseExitingOptions || inTestSuite.OptionGroups.Count == 0)
            {
                inTestSuite.OptionGroups.Clear();
                GetOptionGroups(inTestSuite.OptionGroups, inTestSuite.Components, inTestSuite.NWay);
            }
            inTestSuite.SetUnmarkedOptionGroups();

            Stopwatch timer1 = new Stopwatch();
            timer1.Start();
            while (inTestSuite.UnmarkedOptionGroups.Count != 0)
            {
                // Create 50 single test TestSuites with one option in each component
                var testSuite50 = CreateCandidateTests(inTestSuite);

                // Add the one of the tests that fills the most option groups
                inTestSuite.AddTestCase(FindTestWithMostCoverage(inTestSuite, testSuite50));
            }
            timer1.Stop();
            inTestSuite.CreationTime = timer1.Elapsed;
        }

        public static void GetOptionGroups(OptionGroups inGroups, IEnumerable<SortedTuple> inComponents, int inNWay, SortedTuple inTouple = null, int inComponentId = 0)
        {
            for (int compId = inComponentId; compId < inComponents.Count(); compId++)
            {
                if (inComponents != null)
                {
                    var comp = inComponents.ElementAt(compId);
                    foreach (int option in comp)
                    {
                        var mst = new SortedTuple(inTouple);
                        mst.Add(option);
                        if (mst.Count == inNWay)
                            inGroups.Add(mst, false);
                        else
                            GetOptionGroups(inGroups, inComponents, inNWay, mst, compId + 1);
                    }
                }
            }
        }

        private static TestSuite CreateCandidateTests(TestSuite inTestSuite, int inCandidateCount = 50)
        {
            var testSuite = new TestSuite() { NWay = inTestSuite.NWay };

            for (int i = 0; i < inCandidateCount; i++)
            {
                // Pick a random OptionGroup
                int groupId = RNG.Instance.Next(0, inTestSuite.UnmarkedOptionGroups.Count - 1);

                var testCase = new TestCase();
                testCase.AddRange(inTestSuite.UnmarkedOptionGroups[groupId]);

                foreach (var component in inTestSuite.Components)
                {
                    // Only add a number that is not already added
                    if (testCase.Count(num => component.Contains(num)) > 0)
                        continue;

                    // Find the best number to add
                    var tmpSuite = new TestSuite() { NWay = inTestSuite.NWay };
                    for (int j = 0; j < component.Count; j++)
                    {
                        var num = component[j];
                        var tmp = new TestCase();
                        tmp.AddRange(testCase);
                        tmp.GetPotentialOptionGroups(num, testSuite.NWay);
                        tmp.Add(num); // The options should only have pairs with this item.
                        tmpSuite.AddTestCase(tmp);
                    }
                    testCase = FindTestWithMostCoverage(inTestSuite, tmpSuite);


                    //int compId = RNG.Instance.Next(0, component.Count - 1);
                    //testCase.Add(component[compId]);
                }

                testSuite.AddTestCase(testCase, false);
            }
            return testSuite;
        }

        private static TestCase FindTestWithMostCoverage(TestSuite inTestSuite, TestSuite inTestSubset)
        {
            int bestCount = 0;
            TestCase bestTestcase = null;

            foreach (var testCase in inTestSubset.Tests)
            {
                // Count groups that are not already marked
                int count = testCase.OptionGroups.Count;
                int counter = count;
                foreach (var key in testCase.OptionGroups.Keys)
                {
                    if (inTestSuite.OptionGroups[key])
                        counter--;
                }
                // If not a single item was removed, it is tied for the most optimal
                // so just use the first one found for speed reasons
                if (count == counter)
                    return testCase;

                if (count < bestCount) continue;
                bestCount = count;
                bestTestcase = testCase;
            }
            return bestTestcase;
        }
    }
}
