﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using CoveringArrayAlgorithm;

namespace CoveringArrayAlgorithm_Tests
{
    [TestClass]
    public class SortedTuple_Test
    {
        [TestMethod]
        public void Constructor_NoParams_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);
            
            Assert.AreEqual(5,t1.Count);
        }

        [TestMethod]
        public void Constructor_InSortedTouple_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);

            var t2 = new SortedTuple(t1);
            
            Assert.AreEqual(t1, t2);
        }

        [TestMethod]
        public void Constructor_InSortedTouple_Null_Test()
        {
            SortedTuple t1 = null;

            var t2 = new SortedTuple(t1);

            Assert.AreEqual(t2.Count, 0);
        }

        [TestMethod]
        public void Equals_Null_Right_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);

            bool expected = false;
            bool actual = t1 == null;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void Equals_Null_Left_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);

            SortedTuple t2 = null;

            bool expected = false;
            bool actual = t2 == t1;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void Not_Equals_Null_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);

            bool expected = true;
            bool actual = t1 != null;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void Equals_Self_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);

            var t2 = t1;
            bool expected = true;
            bool actual = t1 == t2;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void Not_Equals_Self_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);

            var t2 = t1;

            bool expected = false;
            bool actual = t1 != t2;
            Assert.AreEqual(expected, actual);
        }
        

        [TestMethod]
        public void Equals_AddSameValues_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);

            var t2 = new SortedTuple();
            t2.Add(1);
            t2.Add(2);
            t2.Add(3);
            t2.Add(4);
            t2.Add(5);

            var expected = true;
            var actual1 = t1 == t2;
            Assert.AreEqual(expected, actual1);
            var actual2 = t1.Equals(t2);
            Assert.AreEqual(expected, actual2);
        }

        [TestMethod]
        public void Equals_UnsortedAdd_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(5);
            t1.Add(3);
            t1.Add(2);
            t1.Add(1);
            t1.Add(4);

            var t2 = new SortedTuple();
            t2.Add(1);
            t2.Add(5);
            t2.Add(3);
            t2.Add(2);
            t2.Add(4);

            var expected = true;
            var actual1 = t1 == t2;
            Assert.AreEqual(expected, actual1);
            var actual2 = t1.Equals(t2);
            Assert.AreEqual(expected, actual2);
        }

        [TestMethod]
        public void Equals_AddUnevenNumbers_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(3);
            t1.Add(2);
            t1.Add(1);
            t1.Add(4);

            var t2 = new SortedTuple();
            t2.Add(1);
            t2.Add(5);
            t2.Add(3);
            t2.Add(2);
            t2.Add(4);

            var expected = false;
            var actual1 = t1 == t2;
            Assert.AreEqual(expected, actual1);
            var actual2 = t1.Equals(t2);
            Assert.AreEqual(expected, actual2);
        }

        [TestMethod]
        public void Not_Equals_Add_DifferentAmountsOfNumbers_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(3);
            t1.Add(2);
            t1.Add(1);
            t1.Add(4);

            var t2 = new SortedTuple();
            t2.Add(1);
            t2.Add(5);
            t2.Add(3);
            t2.Add(2);
            t2.Add(4);

            var expected1 = true;
            var actual1 = t1 != t2;
            Assert.AreEqual(expected1, actual1);

            var expected2 = false;
            var actual2 = t1.Equals(t2);
            Assert.AreEqual(expected2, actual2);
        }

        [TestMethod]
        public void Equals_AddDifferentNumbers_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(3);
            t1.Add(2);
            t1.Add(1);
            t1.Add(4);
            t1.Add(6);

            var t2 = new SortedTuple();
            t2.Add(1);
            t2.Add(5);
            t2.Add(3);
            t2.Add(2);
            t2.Add(4);

            var expected = false;
            var actual1 = t1 == t2;
            Assert.AreEqual(expected, actual1);
            var actual2 = t1.Equals(t2);
            Assert.AreEqual(expected, actual2);
        }

        [TestMethod]
        public void ToString_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(3);
            t1.Add(2);
            t1.Add(1);
            t1.Add(4);
            t1.Add(5);

            string expected = "1, 2, 3, 4, 5";
            string actual = t1.ToString();
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void Item_Iterator_Set_Get_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(3);
            t1.Add(2);
            t1.Add(1);
            t1.Add(4);
            t1.Add(5);

            var t2 = new SortedTuple();
            t2.Add(100);
            t2.Add(200);
            t2.Add(300);
            t2.Add(400);
            t2.Add(500);

            t1[0] = t2[0];
            t1[1] = t2[1];
            t1[2] = t2[2];
            t1[3] = t2[3];
            t1[4] = t2[4];

            Assert.AreEqual(t1,t2);
        }

        [TestMethod]
        public void GetHashCode_Same_Values_Different_Object_Match_Test()
        {
            var t1 = new SortedTuple();
            t1.Add(1);
            t1.Add(2);
            t1.Add(3);
            t1.Add(4);
            t1.Add(5);

            var t2 = new SortedTuple();
            t2.Add(1);
            t2.Add(2);
            t2.Add(3);
            t2.Add(4);
            t2.Add(5);

            Assert.AreEqual(t1.GetHashCode(), t2.GetHashCode());
        }
    }
}
