﻿using System;
using System.IO;
using CoveringArrayView.Model;

namespace CoveringArrayView.Business
{
    /// <summary>
    /// Finds and reads in a TypeCollection from a folder containing one xml per type.
    /// Examples: 
    ///  c:\programdata\UnitTestGenerator\Config\System.Boolean.xml
    ///  c:\programdata\UnitTestGenerator\Config\System.String.xml
    /// 
    /// etc...
    /// </summary>
    public class KnownTypeFileHelper
    {
        public static KnownTypeCollection LoadKnownTypesFromXml()
        {
            var collection = new KnownTypeCollection();

            // Make sure directory exists
            if (!Directory.Exists(XmlDirectory))
                Directory.CreateDirectory(XmlDirectory);

            // Load the XML files
            string[] xmlFiles = Directory.GetFiles(XmlDirectory, "*.xml");
            foreach (var file in xmlFiles)
            {
                // Todo: Verify each file is valid in a better way than try/catch
                try
                {
                    // Deserialize the KnownType from each file and add to the list
                    var knownType = Serializer.DeserializeFromXML<KnownType>(file);
                    knownType.TestValues.Fix();
                    collection.Add(knownType);
                }
                catch (Exception)
                {
                    // Todo: Log an error
                }
            }
            return collection;
        }

        public static string XmlDirectory
        {
            get { return _XmlDirectory ?? Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), "UnitTestGenerator", "Config"); }
            set { _XmlDirectory = value; }
        } private static string _XmlDirectory;


        public static void Serialize(KnownType inKnownType)
        {
            string path = Path.Combine(XmlDirectory, inKnownType.TypeName + ".xml");
            Serializer.SerializeToXML(inKnownType, path);
        }
    }
}
