﻿using System;
using System.Xml.Serialization;
using CoveringArrayView.Extensions;

namespace CoveringArrayView.Model
{
    [Serializable]
    public class TestValue
    {
        private KnownType _Parent;

        public TestValue()
        {
        }

        public TestValue(KnownType inParent)
        {
            //if (inParent == null) throw new NullReferenceException("Parent KnownType cannot be null.");

            SetParent(inParent);
        }

        [XmlIgnore]
        public int Id { get; set; }

        public String Name { get; set; }

        public String Value { get; set; }

        public Option StubCodeType { get; set; }

        public TestValue Clone()
        {
            var tv = new TestValue(_Parent);
            tv.Id = this.Id;
            tv.Name = this.Name;
            tv.Value = this.Value;
            tv.StubCodeType = this.StubCodeType;
            return tv;
        }

        public enum Option
        {
            Code,
            String
        }

        public string GetValueAsCode()
        {
            switch (StubCodeType)
            {
                case Option.String:
                    return Value.HandleQuotes();
                case Option.Code:
                    if (Value.Contains("%"))
                    {
                        return ReplaceTokens();
                    }
                    return Value;
            }
            throw new ApplicationException("Invalid value for the Option enum.");
        }

        private string ReplaceTokens()
        {
            string retStr = Value;
            // Supported tokens
            // %name% = The variable or object short name.

            retStr = retStr.Replace("%name%", _Parent.ShortTypeName);
            retStr = retStr.Replace("%new%", "new " + _Parent.ShortTypeName + "()");

            return retStr;
        }


        public void SetParent(KnownType inParent)
        {
            _Parent = inParent;
        }
    }
}
