﻿using CoveringArrayView.Business;
using CoveringArrayView.Model;

namespace CoveringArrayView.Singleton
{
    /// <summary>
    /// DataStore is basically a memory database to contain 
    /// static objects.
    /// It is a singleton
    /// </summary>
    public class DataStore
    {
        /// <summary>
        /// Only one DataStore can exists (singleton) so the
        /// constructor is private
        /// </summary>
        private DataStore()
        {
            NWay = 2;
            TestFramework = "MSTest";
            UseAAA = true;
        }

        /// <summary>
        /// The single allowed instance of this object
        /// </summary>
        public static DataStore Instance
        {
            get { return _Instance ?? (_Instance = new DataStore()); }
            set { _Instance = value; }
        } private static DataStore _Instance;

        public KnownTypeCollection KnownTypeCollection
        {
            get { return _KnownTypeCollection ?? (_KnownTypeCollection = KnownTypeFileHelper.LoadKnownTypesFromXml()); }
            set { _KnownTypeCollection = value; }
        }private KnownTypeCollection _KnownTypeCollection;

        public int NWay { get; set; }

        /// <summary>
        /// Options should be MSTest for now. 
        /// Todo: Add NUnit
        /// </summary>
        public string TestFramework { get; set; }

        /// <summary>
        /// Use Arrange Act Assert in the stub code
        /// Todo: Add this as an option in the UI 
        /// </summary>
        public bool UseAAA { get; set; }

    }
}
