﻿using System;
using System.Windows.Input;
using CoveringArrayAlgorithm;
using CoveringArrayView.Business;
using CoveringArrayView.Singleton;
using MVVM;
using CoveringArrayView.Model;

namespace CoveringArrayView.ViewModel
{
    class FunctionViewModel : ViewModelBase
    {
        #region Member Variables
        private Function _Function;
        #endregion

        #region Constructors
        /// <summary>
        /// The default constructor
        /// </summary>
        public FunctionViewModel()
        {
        }
        #endregion

        #region Properties
        public String FunctionSignature
        {
            get { return _FunctionSignature; }
            set
            {
                TestSuite.NextOptionId = 0;
                _FunctionSignature = value;
                _Function = FunctionParser.Parse(_FunctionSignature);
                NotifyPropertyChanged("FunctionSignature");
            }
        } private String _FunctionSignature;

        public string StubCode
        {
            get { return _StubCode; }
            set
            {
                _StubCode = value;
                NotifyPropertyChanged("StubCode");
            }
        } private string _StubCode;

        #endregion

        #region ICommands
        public ICommand GenerateStubCodeCommand
        {
            get { return _GenerateStubCodeCommand ?? (_GenerateStubCodeCommand = new RelayCommand(f => GenerateStubCode(), f => GenerateStubCodeCommandCanExecute())); }
            set
            {
                _GenerateStubCodeCommand = value;
                NotifyPropertyChanged("GenerateStubCodeCommand");
            }
        } private ICommand _GenerateStubCodeCommand;
        #endregion

        #region Functions
        private void GenerateStubCode()
        {
            TestSuite.NextOptionId = 0;
            var testSuite = new TestSuite();
            testSuite.NWay = (_Function.Parameters.Count >= DataStore.Instance.NWay)
                ? DataStore.Instance.NWay : _Function.Parameters.Count;

            // Get the Array of Types
            foreach (var param in _Function.Parameters)
            {
                testSuite.Components.Add(new ComponentParameter(param));
                //if (param.KnownType.TestValues.Count < testSuite.NWay)
                //    testSuite.NWay = param.KnownType.TestValues.Count;
            }
            TestSuiteCreator.CreateTestSuite(testSuite);
            //_Function.Parameters

            StubCode = StubCodeBuilder.CreateStubTestMethods(testSuite, _Function);

            // Todo: Get the array of TestValues from each type
            // Todo: Map each test value to a number representation
            // Todo: Use TestSuiteCreator to create a coverying array TestSuite using the number 
            // Todo: Convert the TestSuite back to the parameter types and their test values.
            // Todo:  Create a UnitTest stub for each test case in the TestSuite
        }

        private bool GenerateStubCodeCommandCanExecute()
        {
            return !String.IsNullOrWhiteSpace(FunctionSignature) && _Function != null;
        }
        #endregion
    }
}
