﻿using System.Windows.Input;
using CoveringArrayView.Business;
using CoveringArrayView.Model;
using CoveringArrayView.Singleton;
using MVVM;

namespace CoveringArrayView.ViewModel
{
    public class ManageKnownTypesViewModel : ViewModelBase
    {
        #region Properties
        public KnownTypeCollection KnownTypes
        {
            get { return DataStore.Instance.KnownTypeCollection; }
        }

        public int SelectedKnownTypeId
        {
            get { return _SelectedKnownTypeId; }
            set
            {
                _SelectedKnownTypeId = value;
                NotifyPropertyChanged("SelectedKnownTypeId");
            }
        } private int _SelectedKnownTypeId;

        public KnownType SelectedKnownType
        {
            get { return _SelectedKnownType; }
            set
            {
                _SelectedKnownType = value;
                NotifyPropertyChanged("SelectedKnownType");
            }
        } private KnownType _SelectedKnownType;

        public int SelectedTestValueId
        {
            get { return _SelectedTestValueId; }
            set
            {
                _SelectedTestValueId = value;
                NotifyPropertyChanged("SelectedTestValueId");
            }
        } private int _SelectedTestValueId;

        public TestValue SelectedTestValue
        {
            get { return _SelectedTestValue; }
            set
            {
                _SelectedTestValue = value;
                NotifyPropertyChanged("SelectedTestValue");
            }
        } private TestValue _SelectedTestValue;
        #endregion

        #region ICommands
        public ICommand AddKnownTypeCommand
        {
            get { return _AddKnownTypeCommand ?? (_AddKnownTypeCommand = new RelayCommand(f => AddKnownType())); }
            set
            {
                _AddKnownTypeCommand = value;
                NotifyPropertyChanged("AddKnownTypeCommand");
            }
        } private ICommand _AddKnownTypeCommand;

        public ICommand RemoveKnownTypeCommand
        {
            get { return _RemoveKnownTypeCommand ?? (_RemoveKnownTypeCommand = new RelayCommand(f => RemoveKnownType())); }
            set
            {
                _RemoveKnownTypeCommand = value;
                NotifyPropertyChanged("RemoveKnownTypeCommand");
            }
        } private ICommand _RemoveKnownTypeCommand;

        public ICommand AddValueCommand
        {
            get { return _AddValueCommand ?? (_AddKnownTypeCommand = new RelayCommand(f => AddValue())); }
            set
            {
                _AddValueCommand = value;
                NotifyPropertyChanged("AddValueCommand");
            }
        } private ICommand _AddValueCommand;

        public ICommand RemoveValueCommand
        {
            get { return _RemoveValueCommand ?? (_RemoveValueCommand = new RelayCommand(f => RemoveValue())); }
            set
            {
                _RemoveValueCommand = value;
                NotifyPropertyChanged("RemoveValueCommand");
            }
        } private ICommand _RemoveValueCommand;


        public ICommand SaveCommand
        {
            get { return _SaveCommand ?? (_SaveCommand = new RelayCommand(f => Save())); }
            set
            {
                _SaveCommand = value;
                NotifyPropertyChanged("SaveCommand");
            }
        } private ICommand _SaveCommand;
        #endregion

        #region Functions
        private void AddKnownType()
        {
            DataStore.Instance.KnownTypeCollection.Add(new KnownType() { TypeName = "New Type" });
        }

        private void RemoveKnownType()
        {
            int id = SelectedKnownTypeId;
            DataStore.Instance.KnownTypeCollection.Remove(SelectedKnownType);
            if (DataStore.Instance.KnownTypeCollection.Count > 0)
                SelectedKnownTypeId = DataStore.Instance.KnownTypeCollection.Count - 1;
        }

        private void AddValue()
        {
            SelectedKnownType.TestValues.Add(new TestValue(SelectedKnownType) { Name = "New Value Map" });
            NotifyPropertyChanged("SelectedKnownType");
        }

        private void RemoveValue()
        {
            int id = SelectedTestValueId;
            SelectedKnownType.TestValues.Remove(SelectedTestValue);
            if (_SelectedKnownType.TestValues.Count > 0)
                SelectedKnownTypeId = _SelectedKnownType.TestValues.Count - 1;
        }

        private void Save()
        {
            foreach (var knownType in KnownTypes)
            {
                KnownTypeFileHelper.Serialize(knownType);
            }
        }

        #endregion
    }
}
